@@ -436,7 +436,7 @@ btrfs_subvolume_perform_action (UDisksFilesystemBTRFS *fs_btrfs,
   udisks_linux_block_object_trigger_uevent_sync (object, UDISKS_DEFAULT_WAIT_TIMEOUT);
 
   /* Complete DBus call. */
-  udisks_filesystem_btrfs_complete_set_label (fs_btrfs, invocation);
+  g_dbus_method_invocation_return_value (invocation, g_variant_new ("()"));
 
 out:
   /* Release the resources */
@@ -519,12 +519,12 @@ btrfs_device_perform_action (UDisksFilesystemBTRFS *fs_btrfs,
       goto out;
     }
 
-  /* Update the interface. */
+  /* Trigger uevent on the filesystem and on the added/removed device */
   udisks_linux_block_object_trigger_uevent_sync (object, UDISKS_DEFAULT_WAIT_TIMEOUT);
-  udisks_linux_filesystem_btrfs_update (l_fs_btrfs, object);
+  udisks_daemon_util_trigger_uevent_sync (daemon, device, NULL, UDISKS_DEFAULT_WAIT_TIMEOUT);
 
   /* Complete DBus call. */
-  udisks_filesystem_btrfs_complete_add_device (fs_btrfs, invocation);
+  g_dbus_method_invocation_return_value (invocation, g_variant_new ("()"));
 
 out:
   /* Release the resources */
@@ -620,7 +620,7 @@ handle_get_subvolumes (UDisksFilesystemBTRFS *fs_btrfs,
                                      UDISKS_OBJECT (object),
                                      BTRFS_POLICY_ACTION_ID,
                                      arg_options,
-                                     N_("Authentication is required to change label for BTRFS volume"),
+                                     N_("Authentication is required to get BTRFS subvolumes"),
                                      invocation);
 
   /* Get the mount point for this volume. */
@@ -660,6 +660,106 @@ out:
   return TRUE;
 }
 
+static gboolean
+handle_get_default_subvolume_id (UDisksFilesystemBTRFS *fs_btrfs,
+                       GDBusMethodInvocation *invocation,
+                       GVariant              *arg_options)
+{
+  UDisksLinuxFilesystemBTRFS *l_fs_btrfs = UDISKS_LINUX_FILESYSTEM_BTRFS (fs_btrfs);
+  UDisksLinuxBlockObject *object = NULL;
+  GError *error = NULL;
+  gchar *mount_point = NULL;
+  guint64 default_id;
+
+  object = udisks_daemon_util_dup_object (l_fs_btrfs, &error);
+  if (! object)
+    {
+      g_dbus_method_invocation_take_error (invocation, error);
+      goto out;
+    }
+
+  /* Get the mount point for this volume. */
+  mount_point = udisks_filesystem_btrfs_get_first_mount_point (fs_btrfs, &error);
+  if (! mount_point)
+    {
+      g_dbus_method_invocation_take_error (invocation, error);
+      goto out;
+    }
+
+  default_id = bd_btrfs_get_default_subvolume_id (mount_point, &error);
+  if (! default_id && error)
+    {
+      g_dbus_method_invocation_take_error (invocation, error);
+      goto out;
+    }
+
+  /* Complete DBus call. */
+  udisks_filesystem_btrfs_complete_get_default_subvolume_id (fs_btrfs, invocation, default_id);
+
+out:
+  /* Release the resources */
+  g_clear_object (&object);
+  g_free (mount_point);
+
+  /* Indicate that we handled the method invocation */
+  return TRUE;
+}
+
+static gboolean
+handle_set_default_subvolume_id (UDisksFilesystemBTRFS *fs_btrfs,
+                       GDBusMethodInvocation *invocation,
+                       guint arg_id,
+                       GVariant              *arg_options)
+{
+  UDisksLinuxFilesystemBTRFS *l_fs_btrfs = UDISKS_LINUX_FILESYSTEM_BTRFS (fs_btrfs);
+  UDisksLinuxBlockObject *object = NULL;
+  GError *error = NULL;
+  UDisksDaemon *daemon;
+  gchar *mount_point = NULL;
+
+  object = udisks_daemon_util_dup_object (l_fs_btrfs, &error);
+  if (! object)
+    {
+      g_dbus_method_invocation_take_error (invocation, error);
+      goto out;
+    }
+
+  daemon = udisks_module_get_daemon (UDISKS_MODULE (l_fs_btrfs->module));
+
+  /* Policy check. */
+  UDISKS_DAEMON_CHECK_AUTHORIZATION (daemon,
+                                     UDISKS_OBJECT (object),
+                                     BTRFS_POLICY_ACTION_ID,
+                                     arg_options,
+                                     N_("Authentication is required to set the default BTRFS subvolume"),
+                                     invocation);
+
+  /* Get the mount point for this volume. */
+  mount_point = udisks_filesystem_btrfs_get_first_mount_point (fs_btrfs, &error);
+  if (! mount_point)
+    {
+      g_dbus_method_invocation_take_error (invocation, error);
+      goto out;
+    }
+
+  if (! bd_btrfs_set_default_subvolume (mount_point, arg_id, NULL, &error))
+    {
+      g_dbus_method_invocation_take_error (invocation, error);
+      goto out;
+    }
+
+  /* Complete DBus call. */
+  udisks_filesystem_btrfs_complete_set_default_subvolume_id (fs_btrfs, invocation);
+
+out:
+  /* Release the resources */
+  g_clear_object (&object);
+  g_free (mount_point);
+
+  /* Indicate that we handled the method invocation */
+  return TRUE;
+}
+
 static gboolean
 handle_create_snapshot (UDisksFilesystemBTRFS  *fs_btrfs,
                         GDBusMethodInvocation  *invocation,
@@ -849,6 +949,8 @@ udisks_linux_filesystem_btrfs_iface_init (UDisksFilesystemBTRFSIface *iface)
   iface->handle_create_subvolume = handle_create_subvolume;
   iface->handle_remove_subvolume = handle_remove_subvolume;
   iface->handle_get_subvolumes = handle_get_subvolumes;
+  iface->handle_get_default_subvolume_id = handle_get_default_subvolume_id;
+  iface->handle_set_default_subvolume_id = handle_set_default_subvolume_id;
   iface->handle_create_snapshot = handle_create_snapshot;
   iface->handle_repair = handle_repair;
   iface->handle_resize = handle_resize;
