@@ -1,7 +1,7 @@
 import dbus
 import os
-import six
 import time
+import uuid
 
 import udiskstestcase
 
@@ -48,7 +48,7 @@ class UdisksPartitionTableTest(udiskstestcase.UdisksTestCase):
         # first try to create partition with name -> should fail because mbr
         # doesn't support partition names
         msg = 'MBR partition table does not support names'
-        with six.assertRaisesRegex(self, dbus.exceptions.DBusException, msg):
+        with self.assertRaisesRegex(dbus.exceptions.DBusException, msg):
             disk.CreatePartition(dbus.UInt64(1024**2), dbus.UInt64(100 * 1024**2), part_type, 'name',
                                         self.no_options, dbus_interface=self.iface_prefix + '.PartitionTable')
 
@@ -291,9 +291,9 @@ class UdisksPartitionTableTest(udiskstestcase.UdisksTestCase):
 
         self.addCleanup(self._remove_format, disk)
 
-        # create partition with xfs format
+        # create partition with ext4 format
         path = disk.CreatePartitionAndFormat(dbus.UInt64(1024**2), dbus.UInt64(100 * 1024**2), '', '',
-                                             self.no_options, 'xfs', self.no_options,
+                                             self.no_options, 'ext4', self.no_options,
                                              dbus_interface=self.iface_prefix + '.PartitionTable')
 
         part = self.bus.get_object(self.iface_prefix, path)
@@ -313,7 +313,7 @@ class UdisksPartitionTableTest(udiskstestcase.UdisksTestCase):
         usage.assertEqual('filesystem')
 
         fstype = self.get_property(part, '.Block', 'IdType')
-        fstype.assertEqual('xfs')
+        fstype.assertEqual('ext4')
 
         # check system values
         part_name = path.split('/')[-1]
@@ -328,7 +328,7 @@ class UdisksPartitionTableTest(udiskstestcase.UdisksTestCase):
         self.assertEqual(sys_start * BLOCK_SIZE, 1024**2)
 
         _ret, sys_fstype = self.run_command('lsblk -d -no FSTYPE /dev/%s' % part_name)
-        self.assertEqual(sys_fstype, 'xfs')
+        self.assertEqual(sys_fstype, 'ext4')
 
     def _have_udftools(self):
         ret, _out = self.run_command('type mkudffs')
@@ -429,7 +429,7 @@ class UdisksPartitionTest(udiskstestcase.UdisksTestCase):
             time.sleep(5)
             part.Delete(self.no_options, dbus_interface=self.iface_prefix + '.Partition')
 
-    def _create_partition(self, disk, start=1024**2, size=100 * 1024**2, fmt='xfs', type=''):
+    def _create_partition(self, disk, start=1024**2, size=100 * 1024**2, fmt='ext4', type=''):
         if fmt:
             path = disk.CreatePartitionAndFormat(dbus.UInt64(start), dbus.UInt64(size), type, '',
                                                  self.no_options, fmt, self.no_options,
@@ -485,7 +485,7 @@ class UdisksPartitionTest(udiskstestcase.UdisksTestCase):
         part = self._create_partition(disk)
         self.addCleanup(self._remove_partition, part)
 
-        self._create_format(part, 'xfs')
+        self._create_format(part, 'ext4')
         self.addCleanup(self._remove_format, part)
 
         # set boot flag (10000000(2), 128(10), 0x80(16))
@@ -559,12 +559,12 @@ class UdisksPartitionTest(udiskstestcase.UdisksTestCase):
         part = self._create_partition(disk)
         self.addCleanup(self._remove_partition, part)
 
-        self._create_format(part, 'xfs')
+        self._create_format(part, 'ext4')
         self.addCleanup(self._remove_format, part)
 
         # first try some invalid guid
         msg = 'org.freedesktop.UDisks2.Error.Failed: .* is not a valid UUID'
-        with six.assertRaisesRegex(self, dbus.exceptions.DBusException, msg):
+        with self.assertRaisesRegex(dbus.exceptions.DBusException, msg):
             part.SetType('aaaa', self.no_options,
                          dbus_interface=self.iface_prefix + '.Partition')
 
@@ -593,12 +593,12 @@ class UdisksPartitionTest(udiskstestcase.UdisksTestCase):
         part = self._create_partition(disk)
         self.addCleanup(self._remove_partition, part)
 
-        self._create_format(part, 'xfs')
+        self._create_format(part, 'ext4')
         self.addCleanup(self._remove_format, part)
 
         # try to set part type to an extended partition type -- should fail
         msg = 'Refusing to change partition type to that of an extended partition'
-        with six.assertRaisesRegex(self, dbus.exceptions.DBusException, msg):
+        with self.assertRaisesRegex(dbus.exceptions.DBusException, msg):
             part.SetType('0x05', self.no_options,
                          dbus_interface=self.iface_prefix + '.Partition')
 
@@ -687,12 +687,12 @@ class UdisksPartitionTest(udiskstestcase.UdisksTestCase):
         part = self._create_partition(disk)
         self.addCleanup(self._remove_partition, part)
 
-        self._create_format(part, 'xfs')
+        self._create_format(part, 'ext4')
         self.addCleanup(self._remove_format, part)
 
         # first try some invalid name (longer than 36 characters)
         msg = 'Max partition name length is 36 characters'
-        with six.assertRaisesRegex(self, dbus.exceptions.DBusException, msg):
+        with self.assertRaisesRegex(dbus.exceptions.DBusException, msg):
             part.SetName('a' * 37, self.no_options,
                          dbus_interface=self.iface_prefix + '.Partition')
 
@@ -710,3 +710,53 @@ class UdisksPartitionTest(udiskstestcase.UdisksTestCase):
         part_name = str(part.object_path).split('/')[-1]
         _ret, sys_name = self.run_command('lsblk -d -no PARTLABEL /dev/%s' % part_name)
         self.assertEqual(sys_name, 'test')
+
+    def test_uuid(self):
+        disk = self.get_object('/block_devices/' + os.path.basename(self.vdevs[0]))
+        self.assertIsNotNone(disk)
+
+        self._create_format(disk, 'gpt')
+        self.addCleanup(self._remove_format, disk)
+
+        u = str(uuid.uuid4())
+        d = dbus.Dictionary(signature='sv')
+        d['partition-uuid'] = u
+        path = disk.CreatePartition(dbus.UInt64(1024**2), dbus.UInt64(100 * 1024**2),
+                                    '', '', d,
+                                    dbus_interface=self.iface_prefix + '.PartitionTable')
+
+        self.udev_settle()
+        part = self.bus.get_object(self.iface_prefix, path)
+        self.assertIsNotNone(part)
+        self.addCleanup(self._remove_partition, part)
+
+        # test dbus properties
+        dbus_uuid = self.get_property(part, '.Partition', 'UUID')
+        dbus_uuid.assertEqual(u)
+        # test flags value from system
+        part_name = str(part.object_path).split('/')[-1]
+        _ret, sys_uuid = self.run_command('lsblk -d -no PARTUUID /dev/%s' % part_name)
+        self.assertEqual(sys_uuid, u)
+
+        self._create_format(part, 'ext4')
+        self.addCleanup(self._remove_format, part)
+
+        # try setting invalid UUID
+        msg = 'Provided UUID is not a valid RFC-4122 UUID'
+        for uu in ['garbage', str(uuid.uuid4()) + 'garbage',
+                   '12345678-zzzz-xxxx-yyyy-567812345678', 'ABCD-EFGH']:
+            with self.assertRaisesRegex(dbus.exceptions.DBusException, msg):
+                part.SetUUID(uu, self.no_options, dbus_interface=self.iface_prefix + '.Partition')
+
+        # set new valid UUID
+        u = str(uuid.uuid4())
+        part.SetUUID(u, self.no_options, dbus_interface=self.iface_prefix + '.Partition')
+        self.udev_settle()
+
+        # test dbus properties
+        dbus_uuid = self.get_property(part, '.Partition', 'UUID')
+        dbus_uuid.assertEqual(u)
+        # test flags value from system
+        part_name = str(part.object_path).split('/')[-1]
+        _ret, sys_uuid = self.run_command('lsblk -d -no PARTUUID /dev/%s' % part_name)
+        self.assertEqual(sys_uuid, u)
