@@ -21,7 +21,7 @@
  */
 
 #ifndef __UDISKS_LINUX_FILESYSTEM_HELPERS_H__
-#define __UDISKS_LINUX_FILESYSTEM_HEPLERS_H__
+#define __UDISKS_LINUX_FILESYSTEM_HELPERS_H__
 
 #include <glib.h>
 #include <glib/gstdio.h>
@@ -41,4 +41,4 @@ gboolean take_filesystem_ownership (const gchar *device,
 G_END_DECLS
 
 
-#endif /* __UDISKS_LINUX_FILESYSTEM_HEPLERS_H__ */
+#endif /* __UDISKS_LINUX_FILESYSTEM_HELPERS_H__ */
