@@ -371,7 +371,8 @@ udisks_linux_mdraid_update (UDisksLinuxMDRaid       *mdraid,
                                                  UDISKS_OBJECT (object),
                                                  sync_action_to_job_id (sync_action),
                                                  0,
-                                                 NULL /* cancellable */);
+                                                 TRUE, /* no_inhibit */
+                                                 NULL  /* cancellable */);
 
           /* Mark the job as not cancellable. It simply has to finish... */
           udisks_job_set_cancelable (UDISKS_JOB (job), FALSE);
@@ -643,6 +644,7 @@ handle_start (UDisksMDRaid           *_mdraid,
                                          UDISKS_OBJECT (object),
                                          "md-raid-start",
                                          caller_uid,
+                                         FALSE,
                                          NULL);
 
   if (job == NULL)
@@ -805,6 +807,7 @@ udisks_linux_mdraid_stop (UDisksMDRaid           *_mdraid,
                                          UDISKS_OBJECT (object),
                                          "md-raid-stop",
                                          caller_uid,
+                                         FALSE,
                                          NULL);
 
   if (job == NULL)
@@ -1026,6 +1029,7 @@ handle_remove_device (UDisksMDRaid           *_mdraid,
                                          UDISKS_OBJECT (object),
                                          "md-raid-remove-device",
                                          caller_uid,
+                                         FALSE,
                                          NULL);
 
   if (job == NULL)
@@ -1170,6 +1174,7 @@ handle_add_device (UDisksMDRaid           *_mdraid,
                                          UDISKS_OBJECT (object),
                                          "md-raid-add-device",
                                          caller_uid,
+                                         FALSE,
                                          NULL);
 
   if (job == NULL)
@@ -1286,6 +1291,7 @@ handle_set_bitmap_location (UDisksMDRaid           *_mdraid,
                                          UDISKS_OBJECT (object),
                                          "md-raid-set-bitmap",
                                          caller_uid,
+                                         FALSE,
                                          NULL);
 
   if (job == NULL)
@@ -1333,6 +1339,7 @@ handle_request_sync_action (UDisksMDRaid           *_mdraid,
   GError *error = NULL;
   const gchar *device_file = NULL;
   UDisksBaseJob *job = NULL;
+  gboolean opt_no_inhibit = FALSE;
 
   object = udisks_daemon_util_dup_object (mdraid, &error);
   if (object == NULL)
@@ -1363,6 +1370,8 @@ handle_request_sync_action (UDisksMDRaid           *_mdraid,
       goto out;
     }
 
+  g_variant_lookup (options, "no-inhibit-lock", "b", &opt_no_inhibit);
+
   raid_device = udisks_linux_mdraid_object_get_device (object);
   if (raid_device == NULL)
     {
@@ -1403,6 +1412,7 @@ handle_request_sync_action (UDisksMDRaid           *_mdraid,
                                          UDISKS_OBJECT (object),
                                          sync_action_to_job_id (sync_action),
                                          caller_uid,
+                                         opt_no_inhibit,
                                          NULL);
 
   if (job == NULL)
