@@ -394,10 +394,11 @@ update_iface (UDisksLinuxMDRaidObject  *object,
     {
       if (!has)
         {
+          gpointer iface = g_steal_pointer (interface_pointer);
+
           g_dbus_object_skeleton_remove_interface (G_DBUS_OBJECT_SKELETON (object),
-                                                   G_DBUS_INTERFACE_SKELETON (*interface_pointer));
-          g_object_unref (*interface_pointer);
-          *interface_pointer = NULL;
+                                                   G_DBUS_INTERFACE_SKELETON (iface));
+          g_object_unref (iface);
         }
     }
 
@@ -557,6 +558,7 @@ raid_device_added (UDisksLinuxMDRaidObject *object,
                    UDisksLinuxDevice       *device)
 {
   gchar *level = NULL;
+  GError *error = NULL;
 
   g_assert (object->sync_action_source == NULL);
   g_assert (object->degraded_source == NULL);
@@ -564,8 +566,14 @@ raid_device_added (UDisksLinuxMDRaidObject *object,
   if (!UDISKS_IS_LINUX_DEVICE (device))
     goto out;
 
-  level = read_sysfs_attr (device->udev_device, "md/level");
-  if (level == NULL || !mdraid_has_redundancy (level))
+  level = udisks_linux_device_read_sysfs_attr (device, "md/level", &error);
+  if (!level)
+    {
+       udisks_warning ("mdraid: %s", error->message);
+       g_error_free (error);
+       goto out;
+    }
+  if (!mdraid_has_redundancy (level))
     goto out;
 
   /* udisks_debug ("start watching %s", g_udev_device_get_sysfs_path (device->udev_device)); */
