@@ -226,8 +226,6 @@ udisks_threaded_job_constructed (GObject *object)
 {
   if (G_OBJECT_CLASS (udisks_threaded_job_parent_class)->constructed != NULL)
     G_OBJECT_CLASS (udisks_threaded_job_parent_class)->constructed (object);
-
-  g_assert (g_thread_supported ());
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -374,7 +372,7 @@ udisks_threaded_job_new (UDisksThreadedJobFunc  job_func,
  * Start the @job. Connect to the #UDisksThreadedJob::threaded-job-completed or
  * #UDisksJob::completed signals to get notified when the job is done.
  *
- * */
+ */
 void
 udisks_threaded_job_start (UDisksThreadedJob *job)
 {
